//-----------------------------------------------------------------------------
// Torque Game Engine 
// Copyright (C) GarageGames.com, Inc.
//-----------------------------------------------------------------------------

if ( isObject( moveMap ) )
   moveMap.delete();
new ActionMap(moveMap);


//------------------------------------------------------------------------------
// Non-remapable binds
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// COGMAC
//------------------------------------------------------------------------------

function escapeFromGame()
{
   MessageBoxQuit( "disconnect();", "doNotQuit();");
}

function doNotQuit()
{
   if( !$GameplayEnabled )
   {
      Canvas.cursorOn();
      return;
   }

   crosshair.setVisible(true);
}

function objectiveComplete( %other )
{
   txtDescribeRole.other = %other;
   txtDescribeRole.setText( "USE THREE WORDS TO DESCRIBE THE BEHAVIOR OF THE" SPC %other @ ":\n(e.g. POLITE, VEGETARIAN, CHEAPSKATE, CRAZY, ETC . . .)" );
   txtDescribeReal.setText( "NOW DESCRIBE WHO YOU THINK THE REAL PERSON CONTROLLING THE" SPC %other SPC "IS:" );
   txtAbuse.setText( "CHECK HERE TO REPORT INAPPROPRIATE" SPC %other SPC "BEHAVIOR." );

   if( dlgTutorial.isVisible() )
   {
      MessageBoxQuit( "disconnect();", "doNotQuit();");
      return;
   }

   ///MessageBoxComplete( "disconnect(ObjectiveComplete);", "doNotQuit();");
   MessageBoxComplete( "ShowSurvey();", "doNotQuit();");
}

function ShowSurvey()
{
   CommandToServer('NotifyStartedSurvey');

   dlgSurvey.setVisible(true);
   txtAdjective1.setVisible(true);
   txtAdjective2.setVisible(true);
   txtAdjective3.setVisible(true);
   txtWhereFrom.setVisible(true);
   txtOccupation.setVisible(true);
   txtBreakfast.setVisible(true);
   txtProblems.setVisible(true);

   $GameplayEnabled = false;

   Canvas.cursorOn();
}

//------------------------------------------------------------------------------
// COGMAC
//------------------------------------------------------------------------------


moveMap.bindCmd(keyboard, "escape", "", "escapeFromGame();");

function showPlayerList(%val)
{
   if (%val)
      PlayerListGui.toggle();
}

moveMap.bind( keyboard, F2, showPlayerList );
moveMap.bind( keyboard, F5, toggleParticleEditor);


//------------------------------------------------------------------------------
// Movement Keys
//------------------------------------------------------------------------------

$movementSpeed = 1; // m/s

function setSpeed(%speed)
{
   if(%speed)
      $movementSpeed = %speed;
}

function moveleft(%val)
{
   $mvLeftAction = %val * $movementSpeed;
}

function moveright(%val)
{
   $mvRightAction = %val * $movementSpeed;
}

function moveforward(%val)
{
   $mvForwardAction = %val * $movementSpeed;
}

function movebackward(%val)
{
   $mvBackwardAction = %val * $movementSpeed;
}

function moveup(%val)
{
   $mvUpAction = %val * $movementSpeed;
}

function movedown(%val)
{
   $mvDownAction = %val * $movementSpeed;
}

function turnLeft( %val )
{
   $mvYawRightSpeed = %val ? $Pref::Input::KeyboardTurnSpeed : 0;
}

function turnRight( %val )
{
   $mvYawLeftSpeed = %val ? $Pref::Input::KeyboardTurnSpeed : 0;
}

function panUp( %val )
{
return;
   $mvPitchDownSpeed = %val ? $Pref::Input::KeyboardTurnSpeed : 0;
}

function panDown( %val )
{
return;
   $mvPitchUpSpeed = %val ? $Pref::Input::KeyboardTurnSpeed : 0;
}

function getMouseAdjustAmount(%val)
{
   // based on a default camera fov of 90'
   return(%val * ($cameraFov / 90) * 0.01);
}

///function yaw(%val)
///{
///   $mvYaw += getMouseAdjustAmount(%val);
///}
///
///function pitch(%val)
///{
///   $mvPitch += getMouseAdjustAmount(%val);
///}

function jump(%val)
{
   $mvTriggerCount2++;
}

//moveMap.bind( keyboard, a, moveleft );
moveMap.bind( keyboard, d, moveright );
moveMap.bind( keyboard, w, moveforward );
moveMap.bind( keyboard, s, movebackward );
moveMap.bind( keyboard, space, jump );
moveMap.bind( mouse, xaxis, yaw );
moveMap.bind( mouse, yaxis, pitch );

//------------------------------------------------------------------------------
// COGMAC
//------------------------------------------------------------------------------

function yaw(%val)
{
   // Don't change pitch while interface is up.
   if( !crosshair.isVisible() ) return;

   if($firstPerson)
   {
      $mvYaw += getMouseAdjustAmount(%val);
      commandToServer('Highlight');
   }
   else if($RightMouse)
   {
      $mvCamYaw += getMouseAdjustAmount(%val);
   }

   // Always ensure crosshair is in the middle of the screen.
   %resolution = PlayGui.getExtent();
   %resX = getWord(%resolution, 0);
   %resY = getWord(%resolution, 1);
   crosshair.setPosition( %resX * 0.5, %resY * 0.5 );
}

function pitch(%val)
{
   if($firstPerson)
   {
      commandToServer('Highlight');
   }

   // Don't change pitch while interface is up.
   if( !crosshair.isVisible() ) return;

   if($firstPerson || $RightMouse)
      $mvPitch += getMouseAdjustAmount(%val);
}

function panLeft( %val )
{
   $mvYawRightSpeed = %val ? $Pref::Input::KeyboardTurnSpeed : 0;
}

function panRight( %val )
{
   $mvYawLeftSpeed = %val ? $Pref::Input::KeyboardTurnSpeed : 0;
}

function camYaw(%val)
{
   if($RightMouse)
      $mvCamYaw += getMouseAdjustAmount(%val);
}

moveMap.bind( keyboard, up, upKey );
moveMap.bind( keyboard, down, panDown );
moveMap.bind( keyboard, left, panLeft );
moveMap.bind( keyboard, right, panRight );

function upKey( %val )
{
   mousemod( %val );
}

function zoomIn( %val )
{
return;
   setZoomSpeed( 1000 );
   zoomInFov( %val );
}

function zoomOut( %val )
{
return;
   setZoomSpeed( 1000 );
   zoomOutFov( %val );
}

function zoomOnce( %val )
{
return;
   if($firstPerson) return;

   if ( %val < 0 )
   {
      setZoomSpeed( 1000 );
      zoomOutFovOnce();
   }   
   if ( %val > 0 ) 
   {
      setZoomSpeed( 1000 );
      zoomInFovOnce();
   }
}

moveMap.bind( keyboard, numpadadd, zoomIn );
moveMap.bind( keyboard, numpadminus, zoomOut );
moveMap.bind( mouse, zaxis, zoomOnce );

function mousemod( %val )
{
   if( !$GameplayEnabled ) return;

   // Ignore right mouse button when interface is up.
   if( !crosshair.isVisible() ) return;

   if( %val > 0 ) $RightMouse = true;
   else $RightMouse = false;

   // Drive the character with the right mouse button.
   if( $firstPerson )
   {
      moveforward(%val);
      if( %val > 0 ) commandToServer('MovePlayerStart');
      else commandToServer('MovePlayerEnd');
   }
}
moveMap.bind( mouse, button1, mousemod );

function mouseFire(%val)
{
   if( !$GameplayEnabled ) return;

   if(%val > 0) commandToServer('SelectObjectFirstPerson');
   $mvTriggerCount0++;
   txtChat.doToggle = true;
}

// Beep sound.

new AudioDescription(AudioPhase)
{
   volume   = 250.0;
   isLooping= false;
   is3D     = false;
   type     = $SimAudioType;
};

new AudioProfile(AudioBeep)
{
   filename = "~/data/sound/cogmac/ding.wav";
   description = "AudioPhase";
   preload = true;
};

// Chat input.

function appendChatText( %c )
{
   if( !$GameplayEnabled ) return;

   txtChat.isDeleting = false;

   // Don't chat while flying camera around.
   if( $ChatEnabled != true ) return;

   %resolution = PlayGui.getExtent();
   %resX = getWord(%resolution, 0);
   %resY = getWord(%resolution, 1);

   if( !bmpChatText.isVisible() )
   {
      bmpChatText.setPosition( 0, %resY - 66 );
      bmpChatText.setExtent( %resX, 66 );
      bmpChatText.setVisible( true );

      txtChat.setPosition( 0, %resY - 66 - 8 );
      txtChat.setVisible(true);

      txtChatSmall.setPosition( 0, %resY - 66 - 8 );
      txtChatSmall.setVisible(false);

      txtChat.doToggle = true;
   }

   if( txtChat.doToggle )
   {
      commandToServer('toggleTypingIndicator', true);
      txtChat.doToggle = false;
   }

   %text = txtChat.getText();
   if( txtChat.getTextWidth() > %resX - 32 ) 
   {
      // Switch to smaller font.
      txtChat.setVisible(false);
      txtChatSmall.setVisible(true);
   }

   txtChat.setText( %text @ %c );
   txtChatSmall.setText( %text @ %c );

   if( txtChatSmall.getTextWidth() > %resX ) 
   {
      // Play a ding sound.
      $SoundPlaying = alxPlay(AudioBeep);
      txtChat.setText( %text );
      txtChatSmall.setText( %text );
   }
}

function sendChatText()
{
   %text = txtChat.getText();
   commandToServer('messageSent', %text);
   txtChat.setText( "" );
   txtChatSmall.setText( "" );
   bmpChatText.setVisible( false );
   commandToServer('toggleTypingIndicator', false);
   txtNoListenerWarning.setVisible(false);
}

function toggleDelChatText( %val )
{
   if( %val > 0 )
   {      
      txtChat.isDeleting = true;      
      txtChat.iRepeat = txtChat.iRepeat + 1;
      delChatText( 0 );
   }
   else txtChat.isDeleting = false;
}

function delChatText( %iRepeat )
{
   if( !txtChat.isDeleting ) return;

   // First repeat engages the repeating mechanism.
   // Repeating is cancelled whenever someone hits the key again.
   if( (%iRepeat > 0) && (%iRepeat != txtChat.iRepeat) )
   {
      return;
   }

   %text = txtChat.getText();
   %len = strlen( %text );
   txtChat.setText(  getSubStr( %text, 0, %len - 1 ) );
   txtChatSmall.setText( txtChat.getText() );
   if( %len - 1 <= 0 )
   {
      bmpChatText.setVisible( false );
      commandToServer('toggleTypingIndicator', false);
      txtNoListenerWarning.setVisible(false);
   }

   if( %iRepeat > 0 ) schedule( 100, 0, delChatText, %iRepeat );
   else schedule( 500, 0, delChatText, txtChat.iRepeat );
}

//------------------------------------------------------------------------------
// COGMAC
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
// Mouse Trigger
//------------------------------------------------------------------------------

///function mouseFire(%val)
///{
///   $mvTriggerCount0++;
///}

function altTrigger(%val)
{
   $mvTriggerCount1++;
}

moveMap.bind( mouse, button0, mouseFire );
//moveMap.bind( mouse, button1, altTrigger );


//------------------------------------------------------------------------------
// Zoom and FOV functions
//------------------------------------------------------------------------------

if($Pref::player::CurrentFOV $= "")
   $Pref::player::CurrentFOV = 45;

function setZoomFOV(%val)
{
   if(%val)
      toggleZoomFOV();
}
      
function toggleZoom( %val )
{
   if ( %val )
   {
      $ZoomOn = true;
      setFov( $Pref::player::CurrentFOV );
   }
   else
   {
      $ZoomOn = false;
      setFov( $Pref::player::DefaultFov );
   }
}

moveMap.bind(keyboard, r, setZoomFOV);
moveMap.bind(keyboard, e, toggleZoom);


//------------------------------------------------------------------------------
// Camera & View functions
//------------------------------------------------------------------------------

function toggleFreeLook( %val )
{
   if ( %val )
      $mvFreeLook = true;
   else
      $mvFreeLook = false;
}

$firstPerson = true;
function toggleFirstPerson(%val)
{
return;
   if (%val)
   {
      $firstPerson = !$firstPerson;
      ServerConnection.setFirstPerson($firstPerson);
   }
}

function toggleCamera(%val)
{
   //-------------------------------------------------------------
   // COGMAC
   //-------------------------------------------------------------

   if (%val)
   {
      if( $ChatEnabled )
      {
         $ChatEnabled = false;
         moveMap.bind( keyboard, a, moveleft );
         moveMap.bind( keyboard, d, moveright );
         moveMap.bind( keyboard, w, moveforward );
         moveMap.bind( keyboard, s, movebackward );
      }
      else {
         $ChatEnabled = true;
         moveMap.bindCmd(keyboard, "a", "appendChatText(a);", "");
         moveMap.bindCmd(keyboard, "d", "appendChatText(d);", "");
         moveMap.bindCmd(keyboard, "w", "appendChatText(w);", "");
         moveMap.bindCmd(keyboard, "s", "appendChatText(s);", "");
      }
   }

   //-------------------------------------------------------------
   // COGMAC
   //-------------------------------------------------------------

   if (%val)
      commandToServer('ToggleCamera');
}

//moveMap.bind( keyboard, z, toggleFreeLook );
//moveMap.bind(keyboard, tab, toggleFirstPerson );
//moveMap.bind(keyboard, "alt c", toggleCamera);


//------------------------------------------------------------------------------
// Misc. Player stuff
//------------------------------------------------------------------------------

moveMap.bindCmd(keyboard, "ctrl w", "commandToServer('playCel',\"wave\");", "");
moveMap.bindCmd(keyboard, "ctrl s", "commandToServer('playCel',\"salute\");", "");
moveMap.bindCmd(keyboard, "ctrl k", "commandToServer('suicide');", "");


//------------------------------------------------------------------------------
// Item manipulation
//------------------------------------------------------------------------------

moveMap.bindCmd(keyboard, "h", "commandToServer('use',\"HealthKit\");", "");
moveMap.bindCmd(keyboard, "1", "commandToServer('use',\"Crossbow\");", "");

//------------------------------------------------------------------------------
// Message HUD functions
//------------------------------------------------------------------------------

function pageMessageHudUp( %val )
{
   if ( %val )
      pageUpMessageHud();
}

function pageMessageHudDown( %val )
{
   if ( %val )
      pageDownMessageHud();
}

function resizeMessageHud( %val )
{
   if ( %val )
      cycleMessageHudSize();
}

moveMap.bind(keyboard, u, toggleMessageHud );
//moveMap.bind(keyboard, y, teamMessageHud );
moveMap.bind(keyboard, "pageUp", pageMessageHudUp );
moveMap.bind(keyboard, "pageDown", pageMessageHudDown );
moveMap.bind(keyboard, "p", resizeMessageHud );


//------------------------------------------------------------------------------
// Demo recording functions
//------------------------------------------------------------------------------

function startRecordingDemo( %val )
{
   if ( %val )
      startDemoRecord();
}

function stopRecordingDemo( %val )
{
   if ( %val )
      stopDemoRecord();
}

moveMap.bind( keyboard, F3, startRecordingDemo );
moveMap.bind( keyboard, F4, stopRecordingDemo );


//------------------------------------------------------------------------------
// Helper Functions
//------------------------------------------------------------------------------

function dropCameraAtPlayer(%val)
{
   if (%val)
      commandToServer('dropCameraAtPlayer');
}

function dropPlayerAtCamera(%val)
{
   if (%val)
      commandToServer('DropPlayerAtCamera');
}

moveMap.bind(keyboard, "F8", dropCameraAtPlayer);
moveMap.bind(keyboard, "F7", dropPlayerAtCamera);

function bringUpOptions(%val)
{
   if (%val)
      Canvas.pushDialog(OptionsDlg);
}

moveMap.bind(keyboard, "ctrl o", bringUpOptions);


//------------------------------------------------------------------------------
// Dubuging Functions
//------------------------------------------------------------------------------

$MFDebugRenderMode = 0;
function cycleDebugRenderMode(%val)
{
   if (!%val)
      return;

   if (getBuildString() $= "Debug")
   {
      if($MFDebugRenderMode == 0)
      {
         // Outline mode, including fonts so no stats
         $MFDebugRenderMode = 1;
         GLEnableOutline(true);
      }
      else if ($MFDebugRenderMode == 1)
      {
         // Interior debug mode
         $MFDebugRenderMode = 2;
         GLEnableOutline(false);
         setInteriorRenderMode(7);
         showInterior();
      }
      else if ($MFDebugRenderMode == 2)
      {
         // Back to normal
         $MFDebugRenderMode = 0;
         setInteriorRenderMode(0);
         GLEnableOutline(false);
         show();
      }
   }
   else
   {
      echo("Debug render modes only available when running a Debug build.");
   }
}

GlobalActionMap.bind(keyboard, "F9", cycleDebugRenderMode);


//------------------------------------------------------------------------------
// Misc.
//------------------------------------------------------------------------------

GlobalActionMap.bind(keyboard, "tilde", toggleConsole);
GlobalActionMap.bindCmd(keyboard, "alt enter", "", "toggleFullScreen();");
///GlobalActionMap.bindCmd(keyboard, "F1", "", "contextHelp();");
moveMap.bindCmd(keyboard, "n", "NetGraph::toggleNetGraph();", "");
GlobalActionMap.bindCmd(keyboard, "F1", "", "Canvas.cursorOn();");
GlobalActionMap.bindCmd(keyboard, "F2", "", "Canvas.cursorOff();");
